/*===========================================================================*
 * RGB_graphics.c                                                            *
 *                                                                           *
 *      graphical interface drawing on RGB buffer                            *
 *                                                                           *
 * EXPORTED PROCEDURES:                                                      *
 *      void RGB_pset(unsigned char* RGB24, int width, int height,           *
 *                    unsigned char R, unsigned char G, unsigned char B,     *
 *                    int x, int y)                                          *
 *      void RGB_circle(unsigned char* RGB24, int width, int height,         *
 *                      unsigned char R, unsigned char G, unsigned char B,   *
 *                      int x0, int y0, int rad)                             *
 *      void RGB_rectangle(unsigned char* RGB24, int width, int height,      *
 *                      unsigned char R, unsigned char G, unsigned char B,   *
 *                      int x1, int y1, int x2, int y2, int rad)             *
 *      void RGB_plot (unsigned char* RGB24, int width, int height,          *
 *                      unsigned char R, unsigned char G, unsigned char B,   *
 *                      double x0, double y0, double rad)                    *
 *                                                            (by A. Miyake) *
 *===========================================================================*/
#include <math.h>

/*====================* 
 * INTERNAL FUNCTIONS *
 *====================*/
static void symmetrical_pset8(unsigned char* RGB24, int width, int height,
			  unsigned char R, unsigned char G, unsigned char B,
			  int x, int y, int x0, int y0);


/*===========================================================================*
 * (EXPORTED PROCEDURE)                                                      *
 *                         RGB_pset                                          *
 *===========================================================================*/
void 
RGB_pset(unsigned char* RGB24, int width, int height,
	 unsigned char R, unsigned char G, unsigned char B, int x, int y)
{
   int k;
   if( x < 0 || x >= width || y < 0 || y >= height)  return;

   k = x + y*width;
   RGB24[3*k] = R; RGB24[3*k+1] = G; RGB24[3*k+2] = B; 
}

/*===========================================================================*
 * (EXPORTED PROCEDURE)                                                      *
 *                         RGB_circle                                        *
 *===========================================================================*/
void  
RGB_circle(unsigned char* RGB24, int width, int height, 
	   unsigned char R, unsigned char G, unsigned char B, 
	   int x0, int y0, int rad)
{
   int x = 0;
   int y = rad;
   int d = 3 - 2*rad;
   while(x < y)
   {
      symmetrical_pset8(RGB24, width, height, R, G, B, x, y, x0, y0);
      if(d < 0) d += 4*x + 6;
      else
      {
         d += 4*(x-y) + 10;
         y--;
      }
      x++;
   }
   if(x == y)
     symmetrical_pset8(RGB24, width, height, R, G, B, x, y, x0, y0);
}     
    
/*===========================================================================*
 * (EXPORTED PROCEDURE)                                                      *
 *                         RGB_rectangle                                     *
 *===========================================================================*/
void 
RGB_rectangle(unsigned char* RGB24, int width, int height,
	      unsigned char R, unsigned char G, unsigned char B,
	      int x1, int y1, int x2, int y2, int rad)
{
   int x, y, max_x, min_x, max_y, min_y;

   if(x2 > x1 ) { max_x = x2; min_x = x1; }
   else         { max_x = x1; min_x = x2; }
   if(y2 > y1 ) { max_y = y2; min_x = y1; }
   else         { max_y = y1; min_y = y2; }
 
   for( x = min_x; x <= max_x; x++ )
   {
      RGB_pset(RGB24, width, height, R, G, B, x, y1);      
      RGB_pset(RGB24, width, height, R, G, B, x, y2);
   }
   for( y = min_y; y <= max_y; y++ )
   {
      RGB_pset(RGB24, width, height, R, G, B, x1, y);      
      RGB_pset(RGB24, width, height, R, G, B, x2, y);
   }
}

/*===========================================================================*
 * (EXPORTED PROCEDURE)                                                      *
 *                         RGB_plot                                          *
 *===========================================================================*/
void 
RGB_plot (unsigned char* RGB24, int width, int height,
	  unsigned char R, unsigned char G, unsigned char B,
	  double x, double y, double rad) 
{
   int i, j;
   for(j = floor(y-rad); j <= ceil(y+rad); j++)
   {
      for(i = floor(x-rad); i <= ceil(x+rad); i++)
      {
         if( (i-x)*(i-x) + (j-y)*(j-y) < rad*rad )
	   RGB_pset(RGB24, width, height, R, G, B, i, j);  
      }
   }
}

/*===========================================================================*
 * (INTERNAL PROCEDURE)                                                      *
 *                         symmetrical_pset8                                 *
 *===========================================================================*/
static void 
symmetrical_pset8(unsigned char* RGB24, int width, int height,
		  unsigned char R, unsigned char G, unsigned char B,
		  int x, int y, int x0, int y0)
{
   RGB_pset(RGB24, width, height, R, G, B, x + x0, y +y0);
   RGB_pset(RGB24, width, height, R, G, B, y + x0, x +y0);
   RGB_pset(RGB24, width, height, R, G, B, -x + x0, y +y0);
   RGB_pset(RGB24, width, height, R, G, B, y + x0, -x +y0);
   RGB_pset(RGB24, width, height, R, G, B, x + x0, -y +y0);
   RGB_pset(RGB24, width, height, R, G, B, -y + x0, x +y0);
   RGB_pset(RGB24, width, height, R, G, B, -x + x0, -y +y0);
   RGB_pset(RGB24, width, height, R, G, B, -y + x0, -x +y0);
}
