/*===========================================================================*
 * image_drawing.c                                                           *
 *                                                                           *
 *      Procedures drawing conoscopic image                                  *
 *                                                                           *
 * EXPORTED PROCEDURES:                                                      *
 *      void ImageDrawing()                                                  *
 *      void image_memory_free()                                             *
 *      void change_brightness(gfloat brightness)                            *
 *      void change_gamma(gfloat gamma)                                      *
 *      void set_animation_image_index(gfloat phi)                           *
 *      void get_nth_Aaxis(gint n)                                           *
 *      void get_nth_Baxis(gint n)                                           *
 *      void get_nth_Caxis(gint n)                                           *
 *                                                            (by A. Miyake) *
 *===========================================================================*/
#include <gtk/gtk.h>
#include <math.h>
#include <string.h>
#include "cono_map.h"
#include "gdk_graphics.h"

/*==================*
 * GLOBAL VARIABLES *   
 *==================*/
GdkPixmap *pixmap = NULL;
GtkWidget* ImageArea;
guchar *RGB24;
guchar image_color_mod[256];
gint current_image_index;
gint image_num;
gint image_size, total_image_size;
gfloat image_radius, total_image_radius;
gfloat gamma0, bright0, speed0, phi0;
gfloat RI, NA;
OpticAxisOrientation Aaxis, Baxis, Caxis;

/*==================*
 * STATIC VARIABLES *
 *==================*/
static GdkPixmap **pix;
static gboolean  *pix_created;
static gfloat current_I0, current_gamma;

/*====================* 
 * INTERNAL FUNCTIONS *
 *====================*/
static void image_drawing_init();
static void set_image_color_mod(gfloat gamma, gfloat bright);
static void get_pixmap(int index);

/*====================* 
 * EXTERNAL FUNCTIONS *
 *====================*/
extern gboolean get_all_RGB(guchar**, gfloat*, gfloat);
extern GtkWidget* create_ImageWindow();
//extern GdkGC *get_gdk_pen(GdkWindow *, gdouble, gdouble, gdouble);
//extern void gdk_circle(GdkDrawable *, GdkGC *, gint, gint, gint, gboolean);
//extern void gdk_plot(GdkDrawable *, GdkGC *, gfloat, gfloat, gfloat);

/*=============* 
 * PROJECTIONS *
 *=============*/
double get_Cono_th (double dis) { return asin(dis); }
double get_Wulff_th (double dis) { return atan(dis)*2; }
double get_Cono_dis (double th) { return sin(th); }
double get_Wulff_dis (double th) { return tan(th/2); }
double (*get_th)(double); 
double (*get_dis)(double); 

/*===========================================================================*
 * (EXPORTED PROCEDURES)                                                     *
 *                         ImageDrawing                                      *
 *===========================================================================*/
void 
ImageDrawing()
{
   GtkWidget* ImageWindow;
   gint n;

   image_drawing_init();
   
   RGB24= (guchar *)g_malloc(sizeof(guchar)*3*image_num*image_size*image_size);
      
   if(get_all_RGB(&RGB24, &bright0, gamma0) == FALSE)
   {
      g_free(RGB24);
      return;
   }

   pix = (GdkPixmap **)g_malloc(sizeof(GdkPixmap *)*image_num);
   pix_created = (gboolean *)g_malloc(sizeof(gboolean)*image_num);

   for(n = 0; n < image_num; n++)
     pix[n] = NULL;

   set_image_color_mod(current_gamma=gamma0, current_I0=bright0);

   ImageWindow = create_ImageWindow();
   ImageArea = gtk_object_get_data(GTK_OBJECT(ImageWindow), "ImageArea");
   
   get_pixmap( current_image_index = 0 );
   pixmap = pix[current_image_index];
   gtk_widget_show_all (ImageWindow);

   gtk_grab_add(GTK_WIDGET(ImageWindow));
}

/*===========================================================================*
 * (EXPORTED PROCEDURES)                                                     *
 *                         image_memory_free                                 *
 *===========================================================================*/
void image_memory_free()
{
   int n;
   for(n=0; n < image_num; n++)
   {
      if(pix[n] != NULL)
	gdk_pixmap_unref(pix[n]);
   }
   g_free(pix);
   g_free(pix_created);
   g_free(RGB24);
}

/*===========================================================================*
 * (EXPORTED PROCEDURES)                                                     *
 *                         change_brightness                                 *
 *===========================================================================*/
void 
change_brightness(gfloat brightness)
{
   set_image_color_mod(current_gamma, current_I0=brightness);

   get_pixmap( current_image_index );
   pixmap = pix[current_image_index];
}

/*===========================================================================*
 * (EXPORTED PROCEDURES)                                                     *
 *                         change_gamma                                      *
 *===========================================================================*/
void 
change_gamma(gfloat gamma)
{
   set_image_color_mod(current_gamma= gamma, current_I0);

   get_pixmap( current_image_index );
   pixmap = pix[current_image_index];
}

/*===========================================================================*
 * (EXPORTED PROCEDURES)                                                     *
 *                         set_animation_image_index                         *
 *===========================================================================*/
void 
set_animation_image_index(gfloat phi)
{
   current_image_index = phi*image_num/360.0;
   if(pix_created[current_image_index] == FALSE )
      get_pixmap( current_image_index );
   pixmap = pix[current_image_index];
}

/*===========================================================================*
 * (EXPORTED PROCEDURES)                                                     *
 *                      get_nth_UniaxialAxis                                 *
 *                      get_nth_BiaxialAxes                                  *
 *===========================================================================*/
void 
get_nth_UniaxialAxis(gint n)
{
   gfloat ph;

   if(drawing_method.drawing_kind == Single)
     Caxis.ph = M_PI/180.0*parameter.c_phi;
   else
   {
      ph = n*2.0*M_PI/parameter.image_num;
      Caxis.ph = ph + M_PI/180.0*parameter.c_phi;
   }

   Caxis.th = M_PI/180.0*parameter.c_theta;

   Caxis.x = get_dis(Caxis.th)*cos(Caxis.ph);
   Caxis.y = get_dis(Caxis.th)*sin(Caxis.ph);
}

void 
get_nth_BiaxialAxes(gint n)
{
   gfloat ph;

   if(drawing_method.drawing_kind == Single)
   {
      Aaxis.ph = M_PI/180.0*parameter.a_phi;
      Baxis.ph = M_PI/180.0*parameter.b_phi;
   } 
   else
   {
      ph = n*2.0*M_PI/parameter.image_num;
      Aaxis.ph = ph + M_PI/180.0*parameter.a_phi;
      Baxis.ph = ph + M_PI/180.0*parameter.b_phi;
   }
   Aaxis.th = M_PI/180.0*parameter.a_theta;
   Baxis.th = M_PI/180.0*parameter.b_theta;

   Aaxis.x = get_dis(Aaxis.th)*cos(Aaxis.ph);
   Aaxis.y = get_dis(Aaxis.th)*sin(Aaxis.ph);
   Baxis.x = get_dis(Baxis.th)*cos(Baxis.ph);
   Baxis.y = get_dis(Baxis.th)*sin(Baxis.ph);

   if((cos(Aaxis.th)*cos(Baxis.th) 
       + sin(Aaxis.th)*sin(Baxis.th)*cos(Aaxis.ph-Baxis.ph)) 
     * cos(parameter.optic_axial_angle * M_PI /180) < 0 )
   {
      Baxis.th = M_PI - Baxis.th;
      Baxis.ph = Baxis.ph += M_PI;
   }
}


/*===========================================================================*
 * (INTERNAL PROCEDURE)                                                      *
 *                         image_drawing_init                                *
 *===========================================================================*/
static void image_drawing_init()
{
   if(drawing_method.drawing_kind == Single)
   {
      parameter.image_num = image_num = 1;
      total_image_size = image_size = parameter.image_size;
   }
   else if(drawing_method.drawing_kind == Circular)
   {
      parameter.image_num = image_num = 360/parameter.sampling_step;
      image_size = parameter.image_size;
      total_image_size 
	= rint(image_size*(1 + 1/sin(M_PI/image_num)) + 0.5);
   }
   else if(drawing_method.drawing_kind == Animation)
   {
      parameter.image_num = image_num = 360/parameter.sampling_step;
      total_image_size = image_size = parameter.image_size;
   }
   image_radius = image_size/2.0;
   total_image_radius = total_image_size/2.0;


   if(drawing_method.drawing_kind == Animation )
   {
      phi0 = 0;
      speed0 = 45;
   }
   gamma0 = 2.4;

   if(drawing_method.projection == WulffProjection)
   {
      get_th = &get_Wulff_th;
      get_dis = &get_Wulff_dis;
   }
   else if (drawing_method.projection == RealProjection)
   {
      get_th = &get_Cono_th;
      get_dis = &get_Cono_dis;
   }

   if(drawing_method.axial_type == Uniaxial)
     RI = sqrt(parameter.omega*parameter.epsilon);
   else
     RI = sqrt(parameter.alpha*parameter.gamma);
   
   NA = parameter.NA;
}

/*===========================================================================*
 * (INTERNAL PROCEDURE)                                                      *
 *                         set_image_color_mod                               *
 *===========================================================================*/
static void set_image_color_mod(gfloat gamma, gfloat bright)
{
   int n, k;
   double I;
   for(k = 0; k < 255; k++)
   {
       I = pow(bright/bright0, 1.0/gamma)*pow(k/254.0, gamma0/gamma);
       image_color_mod[k] = (I >= 1.0) ? 255 : I*255;
   }
   image_color_mod[255] = 255;

   for( n = 0; n < image_num; n++)
     pix_created[n] = FALSE;
}

/*===========================================================================*
 * (INTERNAL PROCEDURE)                                                      *
 *                         get_pixmap                                        *
 *===========================================================================*/
static void get_pixmap( int index )
{
   int k, n;
   guchar *RGB240 = RGB24 + 3*index*image_size*image_size;
   guchar* mod_RGB24=(guchar*)g_malloc(sizeof(guchar)*3*image_size*image_size);
   static int first_call = 1;
   static GdkGC *greenPen, *yellowPen;
   gfloat ax, ay, bx, by, cx, cy;
   gfloat x0, y0, R;
   gint conoscopic_num;
   gfloat visible_radius;

   for(k = 0; k < 3*image_size*image_size; k++)
     mod_RGB24[k] = image_color_mod[RGB240[k]];

   if(pix[index] != NULL )
     gdk_pixmap_unref(pix[index]);

   pix[index] = gdk_pixmap_new(ImageArea->window, image_size, image_size, -1);
   gdk_draw_rgb_image( pix[index],
		      ImageArea->style->fg_gc[GTK_STATE_NORMAL], 
		      0, 0, image_size, image_size, 
		      GDK_RGB_DITHER_MAX, mod_RGB24, image_size*3);
   
   g_free( mod_RGB24);
   pix_created [index] = TRUE;

   if(first_call)
   {
      greenPen = get_gdk_pen(ImageArea->window, 0, 1, 0);
      yellowPen = get_gdk_pen(ImageArea->window, 1, 1, 0);
      first_call = 0;
   }

   if(drawing_method.drawing_range == WholeAngle)
   {
      visible_radius = image_radius*get_dis(asin(NA / RI));
      R = (total_image_size-parameter.image_size)/2;
      conoscopic_num = (drawing_method.drawing_kind == Circular) ?
	parameter.image_num : 1; 
      
      for(n = 0; n < conoscopic_num; n++)
      {
	 x0 = R*cos(2*M_PI*n/parameter.image_num) + R + image_radius - 0.5;
	 y0 = -R*sin(2*M_PI*n/parameter.image_num)+ R + image_radius - 0.5;
	 
	 gdk_circle(pix[index], greenPen, rint(x0), rint(y0), 
		    rint(visible_radius), FALSE);
	 
	 if(drawing_method.axial_type == Uniaxial)
	 {
	    if(drawing_method.drawing_kind == Circular)
	      get_nth_UniaxialAxis(n);
	    else
	      get_nth_UniaxialAxis(index);

	    cx = x0 + parameter.image_size/2 * Caxis.x;
	    cy = y0 - parameter.image_size/2 * Caxis.y;
	    
	    gdk_plot(pix[index], yellowPen, cx, cy, 2.0);
	 }
	 else
	 {
	    if(drawing_method.drawing_kind == Circular)
	      get_nth_BiaxialAxes(n);
	    else
	      get_nth_BiaxialAxes(index);
	    
	    ax = x0 + parameter.image_size/2 * Aaxis.x;
	    ay = y0 - parameter.image_size/2 * Aaxis.y;
	    bx = x0 + parameter.image_size/2 * Baxis.x;
	    by = y0 - parameter.image_size/2 * Baxis.y;
	    
	    gdk_plot(pix[index], yellowPen, ax, ay, 2.0);
	    gdk_plot(pix[index], yellowPen, bx, by, 2.0);
	 }    
      }
   }
}
		      


      
	  

  

