/*===========================================================================*
 * stereo.c                                                                  *
 *                                                                           *
 *      Stereo plot of optic axes and optical elastisity axes                *
 *                                                                           *
 * EXPORTED PROCEDURES:                                                      *
 *      GtkWidget *createStereoFrame(GtkWidget*)                             *
 *      void updateStereoFrame()                                             * 
 *                                                            (by A. Miyake) *
 *===========================================================================*/
#include <gtk/gtk.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include "cono_map.h"
#include "widget_set.h"
#include "gdk_graphics.h"

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include "intl.h"

/*====================*
 * EXTERNAL VARIABLES *   
 *====================*/
extern GtkStyle* GreekStyle;  /* from main.c */
extern GtkTooltips *tooltips; /* from main.c */

/*==================*
 * STATIC VARIABLES *
 *==================*/
static GtkWidget *StereoFrame;
static GtkWidget *StereoArea;
static GdkPixmap *StereoPix = NULL;
static GdkGC *penBlack = NULL;
static GdkGC *penWhite = NULL;
static GdkGC *penGreen = NULL;
static GdkGC *penYellow = NULL;
static GdkGC *penOrange = NULL;
static GdkGC *penRed = NULL;
static GdkGC *penBlue = NULL;
static GdkFont *Font;

static gint stereo_size = 150;
static gint stereo_area_xsize = 220;
static gint stereo_area_ysize = 160;

/*====================*
 * EXTERNAL FUNCTIONS *   
 *====================*/
extern void gtk_button_replace_label(GtkButton* button, gchar* ); 

/*====================* 
 * INTERNAL FUNCTIONS *
 *====================*/
static void projection_change(GtkButton* , gpointer);
static gint StereoAreaExpose(GtkWidget*, GdkEventExpose* );
static void axis_plot(GdkGC* gc, gfloat th, gfloat ph, gint size );
static void draw_plot_legend(gint plot_size);

/*=====================*
 * EXPORTED PROCEDURES *
 *=====================*/
GtkWidget *createStereoFrame();
void updateStereoFrame();

/*===============================================================*
 * (EXPORTED PROCEDURE)                                          *
 *               createStereoFrame                              *
 *===============================================================*/
GtkWidget *createStereoFrame(GtkWidget* parent)
{
   GtkWidget *StereoBox;
   GtkWidget *ProjectionButton;

   StereoFrame = gtk_frame_new(NULL);
   gtk_container_set_border_width( GTK_CONTAINER(StereoFrame), 10);
   gtk_box_pack_start(GTK_BOX(parent), StereoFrame, FALSE, FALSE, 7);

   StereoBox  = gtk_vbox_new(FALSE, 0);
   gtk_container_add(GTK_CONTAINER(StereoFrame), StereoBox);
   
   ProjectionButton = pack_button(StereoBox, _("Conoscopic Projection"));
   gtk_container_set_border_width (GTK_CONTAINER (StereoBox), 0);   
   drawing_method.projection = RealProjection;
   gtk_tooltips_set_tip (tooltips, ProjectionButton, _("Projection"), NULL);
   
   gtk_signal_connect (GTK_OBJECT (ProjectionButton), "clicked",
		       GTK_SIGNAL_FUNC (projection_change), NULL);
   
   StereoArea = gtk_drawing_area_new();
   gtk_drawing_area_size(GTK_DRAWING_AREA(StereoArea), 
			 stereo_area_xsize, stereo_area_ysize);
   gtk_box_pack_start (GTK_BOX(StereoBox), StereoArea, FALSE, FALSE, 0);

   gtk_widget_show_all(gtk_widget_get_toplevel(parent));
   if(StereoPix != NULL )
     gdk_pixmap_unref(StereoPix);
  
   StereoPix = gdk_pixmap_new( StereoArea->window, 
			       StereoArea->allocation.width,
			       StereoArea->allocation.height, -1);

   penBlack = get_gdk_pen(StereoArea->window, 0, 0, 0);
   penWhite = get_gdk_pen(StereoArea->window, 1, 1, 1);
   penGreen = get_gdk_pen(StereoArea->window, 0, 1, 0);
   penYellow = get_gdk_pen(StereoArea->window, 1, 1, 0);
   penOrange = get_gdk_pen(StereoArea->window, 1, 0.5, 0);
   penRed = get_gdk_pen(StereoArea->window, 1, 0, 0);
   penBlue = get_gdk_pen(StereoArea->window, 0.5, 0.5, 1);

   Font = gtk_widget_get_style(StereoArea)->font;
   updateStereoFrame();

   gtk_signal_connect (GTK_OBJECT (StereoArea), "expose_event",
		       GTK_SIGNAL_FUNC (StereoAreaExpose), NULL);
   
   return StereoFrame;
}

/* =================================*
 * (CALLBACK for createStereoFrame) *
 *     projection_change            *
 * =================================*/
static void projection_change(GtkButton* button, gpointer data)
{
   if(drawing_method.projection == RealProjection)
   {
      drawing_method.projection = WulffProjection;
      gtk_button_replace_label(button, _("Wullf Projection"));
   }
   else if(drawing_method.projection == WulffProjection)
   {
      drawing_method.projection = RealProjection;
      gtk_button_replace_label(button, _("Conoscopic Projection"));
   }
   updateStereoFrame();
}

/*===============================================================*
 * (EXPORTED PROCEDURE)                                          *
 *                  updateStereoFrame                            *
 *===============================================================*/
void updateStereoFrame()
{
   GdkRectangle update_rect;
   gint radius = stereo_size/2;
   gint microscopic_radius = 0;
   gint plot_size = 6;

   gdk_rectangle(StereoPix, penBlack, 0, 0, 
		 StereoArea->allocation.width, StereoArea->allocation.height,
		 TRUE );
   gdk_circle(StereoPix, penWhite, stereo_area_ysize/2, stereo_area_ysize/2,
	      stereo_size/2, FALSE );

   if(drawing_method.projection == RealProjection)
     microscopic_radius = rint(radius*parameter.NA/parameter.RI);
   else if(drawing_method.projection == WulffProjection)
     microscopic_radius = rint(radius*tan(asin(parameter.NA/parameter.RI)/2));

   gdk_circle(StereoPix, penGreen, stereo_area_ysize/2, stereo_area_ysize/2,
	      microscopic_radius, FALSE);


   if(drawing_method.axial_type == Uniaxial)
   {
      axis_plot(penYellow, parameter.c_theta, parameter.c_phi, plot_size);
   }
      
   else if(drawing_method.axial_type == Biaxial)
   {
      axis_plot(penYellow, parameter.a_theta, parameter.a_phi, plot_size);
      axis_plot(penYellow, parameter.b_theta, parameter.b_phi, plot_size);
      
      axis_plot(penOrange, parameter.x_theta, parameter.x_phi, plot_size);
      axis_plot(penBlue, parameter.z_theta, parameter.z_phi, plot_size);
      axis_plot(penRed, parameter.y_theta, parameter.y_phi, plot_size);
   }

   draw_plot_legend(plot_size);

   update_rect.x = update_rect.y = 0;
   update_rect.width = StereoArea->allocation.width;
   update_rect.height = StereoArea->allocation.height;

   gtk_widget_draw(StereoArea, &update_rect); 
}

/*=======================================================================*
 * (CALLBACK and related functions for updateSetreoFrame)                *
 *                                                                       *
 *     StereoAreaExpose                                                  *
 *     axis_plot                                                         *
 *     draw_plot_legend                                                  *
 *=======================================================================*/
gint
StereoAreaExpose(GtkWidget *widget, GdkEventExpose *event)
{
   gdk_draw_pixmap(widget->window,
                   widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
                   StereoPix, event->area.x, event->area.y, event->area.x,
                   event->area.y, event->area.width, event->area.height);
   return FALSE;
}
      
static void 
axis_plot(GdkGC* gc, gfloat th, gfloat ph, gint size)
{
   gint x0=0, y0=0, x1, y1, x2, y2;

   if(drawing_method.projection == RealProjection)
   {
      x0 = rint(stereo_size/2*sin(th*M_PI/180)*cos(ph*M_PI/180));
      y0 = rint(stereo_size/2*sin(th*M_PI/180)*sin(ph*M_PI/180));
   }
   else if(drawing_method.projection == WulffProjection)
   {
      x0 = rint(stereo_size/2*tan(th*M_PI/360)*cos(ph*M_PI/180));
      y0 = rint(stereo_size/2*tan(th*M_PI/360)*sin(ph*M_PI/180));
   }
   
   x1 = (stereo_area_ysize-size)/2 + x0; x2 = x1 + size; 
   y1 = (stereo_area_ysize-size)/2 - y0; y2 = y1 + size;
  
   gdk_rectangle(StereoPix, gc, x1, y1, x2, y2, TRUE);
}

static void
draw_plot_legend(gint plot_size)
{
   gint startx = stereo_area_ysize+10;
   gint endy = stereo_area_ysize-plot_size-10;
   gint dy = 15;

   if(drawing_method.axial_type == Uniaxial)
   {
      gdk_draw_rectangle(StereoPix, penYellow, TRUE, 
			 startx, endy, plot_size, plot_size);
      gdk_draw_string(StereoPix, Font, penWhite, 
		      startx+plot_size+5, endy+plot_size,"C");
   }
   else if(drawing_method.axial_type == Biaxial)
   {
      gdk_draw_rectangle(StereoPix, penYellow, TRUE, 
			 startx, endy-3*dy, plot_size, plot_size);
      gdk_draw_string(StereoPix, Font, penWhite, 
		      startx+plot_size+5, endy-3*dy+plot_size,"A B");

      gdk_draw_rectangle(StereoPix, penOrange, TRUE, 
			 startx, endy-2*dy , plot_size, plot_size);
      gdk_draw_string(StereoPix, Font, penWhite, 
		      startx+plot_size+5, endy-2*dy+plot_size,"X");
   
      gdk_draw_rectangle(StereoPix, penRed, TRUE, 
			 startx, endy-dy , plot_size, plot_size);
      gdk_draw_string(StereoPix, Font, penWhite, 
		      startx+plot_size+5, endy-dy+plot_size,"Y");
   
      gdk_draw_rectangle(StereoPix, penBlue, TRUE, 
			 startx, endy, plot_size, plot_size);
      gdk_draw_string(StereoPix, Font, penWhite, 
		      startx+plot_size+5, endy+plot_size,"Z");
   }
}
   
